from __future__ import annotations

import csv
from functools import cached_property
from gzip import GzipFile
from urllib.request import urlopen


class Score(object):
    """ EPSS Score Object"""

    def __init__(self, cve: str, epss: str, percentile: str):
        self.cve = cve
        self.epss = float(epss)
        self.percentile = float(percentile)


class EPSS(object):
    def __init__(self) -> None:
        pass

    def scores(self) -> list[Score]:
        """Get all CVE's EPSS scores (downloaded data is cached in memory)

        Example

        [
        {'cve': 'CVE-2022-39952', 'epss': '0.09029', 'percentile': '0.94031'},
        {'cve': 'CVE-2023-0669', 'epss': '0.78437', 'percentile': '0.99452'},
        ...
        ]

        Returns:
            list[Score]: EPSS score's csv list
        """
        scores = [row for row in csv.DictReader(self._download[1:])]
        return [Score(row['cve'], row['epss'], row['percentile'])
                for row in scores]

    def score(self, cve_id: str) -> Score:
        """Get EPSS score and percentile

        Example
            {'cve': 'CVE-2022-39952', 'epss': 0.0095, 'percentile': 0.32069}

        Args:
            cve_id (str): CVE ID (CVE-nnnn)

        Returns:
            Score | None: EPSS score percentile
        """
        rows = self._filter_by_cve_id(cve_id)
        if len(rows) == 1:
            return rows[0]
        else:
            return None

    def epss(self, cve_id: str) -> float:
        """Get EPSS score

        Args:
            cve_id (str): CVE ID (CVE-nnnn)

        Returns:
            float | None: EPSS score (0.0-1.0)
        """
        rows = self._filter_by_cve_id(cve_id)
        if len(rows) == 1:
            return rows[0].epss
        else:
            return None

    def percentile(self, cve_id: str) -> float:
        """Get EPSS percentile

        Args:
            cve_id (str): CVE ID (CVE-nnnn)

        Returns:
            float | None: EPSS percentile (0.0-1.0)
        """
        rows = self._filter_by_cve_id(cve_id)
        if len(rows) == 1:
            return rows[0].percentile
        else:
            return None

    def epss_gt(self, max: float) -> list[Score]:
        """Get CVEs with EPSS score greater or equal than the parameter

        Args:
            max (float): limit of EPSS score

        Returns:
            list[Score] | None: EPSS score object list
        """
        rows = [r for r in filter(lambda x: x.epss >= max, self.scores())]
        return rows

    def percentile_gt(self, max: float) -> list[Score]:
        """Get CVEs with percentile greater or equal than the parameter

        Args:
            max (float): limit of EPSS score

        Returns:
            list[Score] | None: EPSS score object list
        """
        rows = [r for r in
                filter(lambda x: x.percentile >= max, self.scores())]
        return rows

    def epss_lt(self, min: float) -> list[Score]:
        """Get CVEs with EPSS score lower or equal than the parameter

        Args:
            min (float): limit of EPSS score

        Returns:
            list[Score] | None: EPSS score object list
        """
        rows = [r for r in filter(lambda x: x.epss <= min, self.scores())]
        return rows

    def percentile_lt(self, min: float) -> list[Score]:
        """Get CVEs with percentile lower or equal than the parameter

        Args:
            min (float): limit of EPSS score

        Returns:
            list[Score] | None: EPSS score object list
        """
        rows = [r for r in
                filter(lambda x: x.percentile <= min, self.scores())]
        return rows

    def csv(self) -> list[str]:
        """Get csv data containing all epss scores.

        Example
            #model_version:v2022.01.01,score_date:2023-03-03T00:00:00+0000
            cve,epss,percentile
            CVE-2014-6271,0.96235,0.99992
            CVE-2014-7169,0.83508,0.99607
            ...

        Returns:
            list[str]: csv data
        """
        return self._download

    def _filter_by_cve_id(self, cve_id: str):
        cve_filter = filter(lambda x: x.cve == cve_id, self.scores())
        rows = [row for row in cve_filter]
        return rows

    @cached_property
    def _download(self):
        url = 'https://epss.cyentia.com/epss_scores-current.csv.gz'

        with urlopen(url) as res:
            dec = GzipFile(fileobj=res)
            epss_scores_str: str = dec.read().decode("utf-8")
            epss_scores_list = epss_scores_str.split('\n')

        return epss_scores_list



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['epss_api']

package_data = \
{'': ['*']}

install_requires = \
['urllib3>=1.26.13,<2.0.0']

setup_kwargs = {
    'name': 'epss-api',
    'version': '1.1.3',
    'description': 'EPSS API Python Client',
    'long_description': "=================\nEPSS API Client\n=================\n\n.. image:: https://badge.fury.io/py/epss-api.svg\n    :target: https://badge.fury.io/py/epss-api\n\n.. image:: https://img.shields.io/pypi/dw/epss-api?style=flat\n    :target: https://pypistats.org/packages/epss-api\n\n.. image:: https://github.com/kannkyo/epss-api/actions/workflows/python-ci.yml/badge.svg\n    :target: https://github.com/kannkyo/epss-api/actions/workflows/python-ci.yml\n\n.. image:: https://codecov.io/gh/kannkyo/epss-api/branch/main/graph/badge.svg?token=R40FT0KITO \n    :target: https://codecov.io/gh/kannkyo/epss-api\n\n.. image:: https://github.com/kannkyo/epss-api/actions/workflows/scorecards.yml/badge.svg\n    :target: https://github.com/kannkyo/epss-api/actions/workflows/scorecards.yml\n\nEPSS(Exploit Prediction Scoring System) API client.\n\nEPSS is the one of famous vulnerability score developed by FIRST (the Forum of Incident Response and Security Teams).\n\nEPSS's definition:\n\n    The Exploit Prediction Scoring System (EPSS) is an open, \n    data-driven effort for estimating the likelihood (probability) that a software vulnerability will be exploited in the wild. \n    Our goal is to assist network defenders to better prioritize vulnerability remediation efforts. \n    While other industry standards have been useful for capturing innate characteristics of a vulnerability and provide measures of severity, \n    they are limited in their ability to assess threat. \n    EPSS fills that gap because it uses current threat information from CVE and real-world exploit data. \n    The EPSS model produces a probability score between 0 and 1 (0 and 100%). \n    The higher the score, the greater the probability that a vulnerability will be exploited.\n\n    https://www.first.org/epss/\n\nThis package is most easiest and efficient EPSS api client.\n\nUsage\n=============\n\nEPSS has some methods.\n\n.. code-block:: python\n\n    from epss_api import EPSS\n\n    client = EPSS()\n\n    value = client.scores()\n    # value = [\n    #   {'cve': 'CVE-2022-39952', 'epss': '0.09029', 'percentile': '0.94031'},\n    #   {'cve': 'CVE-2023-0669', 'epss': '0.78437', 'percentile': '0.99452'},\n    #  ...\n    # ]\n\n    value = client.epss_lt(0.5)\n    # value = [\n    #   {'cve': 'CVE-2022-39952', 'epss': '0.09029', 'percentile': '0.24031'},\n    #   {'cve': 'CVE-2023-0669', 'epss': '0.18437', 'percentile': '0.19452'},\n    #  ...\n    # ]\n\n    value = client.percentile_lt(0.5)\n    # value = [\n    #   {'cve': 'CVE-2022-39952', 'epss': '0.09029', 'percentile': '0.24031'},\n    #   {'cve': 'CVE-2023-0669', 'epss': '0.78437', 'percentile': '0.19452'},\n    #  ...\n    # ]\n\n    value = client.epss_gt(0.5)\n    # value = [\n    #   {'cve': 'CVE-2022-39952', 'epss': '0.59029', 'percentile': '0.94031'},\n    #   {'cve': 'CVE-2023-0669', 'epss': '0.78437', 'percentile': '0.99452'},\n    #  ...\n    # ]\n\n    value = client.percentile_gt(0.5)\n    # value = [\n    #   {'cve': 'CVE-2022-39952', 'epss': '0.59029', 'percentile': '0.94031'},\n    #   {'cve': 'CVE-2023-0669', 'epss': '0.78437', 'percentile': '0.99452'},\n    #  ...\n    # ]\n\n    value = client.score(cve_id='CVE-2022-0669')\n    # value = {'cve': 'CVE-2022-39952', 'epss': 0.0095, 'percentile': 0.32069}\n\n    value = client.epss(cve_id='CVE-2022-0669')\n    # value == 0.0095\n\n    value = client.percentile(cve_id='CVE-2022-0669')\n    # value == 0.32069\n\nIf you call either one method, EPSS client cache all CVE's score in memory.\nAfter caching, you can get all data very fast.\n",
    'author': 'kannkyo',
    'author_email': '15080890+kannkyo@users.noreply.github.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/kannkyo/epss-api',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
